@@ -1,4 +1,4 @@
-/*	$OpenBSD: kroute.c,v 1.157 2007/11/24 17:01:04 claudio Exp $ */
+/*	$OpenBSD: kroute.c,v 1.160 2008/05/09 12:45:25 henning Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -1759,6 +1759,7 @@ send_rtmsg(int fd, int action, struct kroute *kroute)
 	hdr.rtm_type = action;
 	hdr.rtm_tableid = kr_state.rtableid;
 	hdr.rtm_flags = RTF_PROTO1;
+	hdr.rtm_priority = RTP_BGP;
 	if (kroute->flags & F_BLACKHOLE)
 		hdr.rtm_flags |= RTF_BLACKHOLE;
 	if (kroute->flags & F_REJECT)
