@@ -1,4 +1,4 @@
-/*	$OpenBSD: mrt.c,v 1.50 2006/02/08 14:49:58 claudio Exp $ */
+/*	$OpenBSD: mrt.c,v 1.53 2007/04/23 13:04:24 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Claudio Jeker <claudio@openbsd.org>
@@ -91,6 +91,7 @@ mrt_dump_bgp_msg(struct mrt *mrt, void *pkg, u_int16_t pkglen,
 {
 	struct buf	*buf;
 	u_int16_t	 len;
+	int		 incoming = 0;
 
 	switch (peer->sa_local.ss_family) {
 	case AF_INET:
@@ -103,6 +104,10 @@ mrt_dump_bgp_msg(struct mrt *mrt, void *pkg, u_int16_t pkglen,
 		return (-1);
 	}
 
+	/* get the direction of the message to swap address and AS fields */
+	if (mrt->type == MRT_ALL_IN || mrt->type == MRT_UPDATE_IN)
+		incoming = 1;
+
 	if ((buf = buf_open(len + MRT_HEADER_SIZE)) == NULL) {
 		log_warnx("mrt_dump_bgp_msg: buf_open error");
 		return (-1);
@@ -114,29 +119,49 @@ mrt_dump_bgp_msg(struct mrt *mrt, void *pkg, u_int16_t pkglen,
 		return (-1);
 	}
 
-	DUMP_SHORT(buf, bgp->as);
-	DUMP_SHORT(buf, peer->conf.remote_as);
+	if (!incoming)
+		DUMP_SHORT(buf, bgp->short_as);
+	DUMP_SHORT(buf, peer->short_as);
+	if (incoming)
+		DUMP_SHORT(buf, bgp->short_as);
 	DUMP_SHORT(buf, /* ifindex */ 0);
 	switch (peer->sa_local.ss_family) {
 	case AF_INET:
 		DUMP_SHORT(buf, AFI_IPv4);
-		DUMP_NLONG(buf,
-		    ((struct sockaddr_in *)&peer->sa_local)->sin_addr.s_addr);
+		if (!incoming)
+			DUMP_NLONG(buf, ((struct sockaddr_in *)
+			    &peer->sa_local)->sin_addr.s_addr);
 		DUMP_NLONG(buf,
 		    ((struct sockaddr_in *)&peer->sa_remote)->sin_addr.s_addr);
+		if (incoming)
+			DUMP_NLONG(buf, ((struct sockaddr_in *)
+			    &peer->sa_local)->sin_addr.s_addr);
 		break;
 	case AF_INET6:
 		DUMP_SHORT(buf, AFI_IPv6);
+		if (!incoming)
+			if (buf_add(buf, &((struct sockaddr_in6 *)
+			    &peer->sa_local)->sin6_addr,
+			    sizeof(struct in6_addr)) == -1) {
+				log_warnx("mrt_dump_bgp_msg: buf_add error");
+				buf_free(buf);
+				return (-1);
+			}
 		if (buf_add(buf,
-		    &((struct sockaddr_in6 *)&peer->sa_local)->sin6_addr,
-		    sizeof(struct in6_addr)) == -1 ||
-		    buf_add(buf,
 		    &((struct sockaddr_in6 *)&peer->sa_remote)->sin6_addr,
 		    sizeof(struct in6_addr)) == -1) {
 			log_warnx("mrt_dump_bgp_msg: buf_add error");
 			buf_free(buf);
 			return (-1);
 		}
+		if (incoming)
+			if (buf_add(buf, &((struct sockaddr_in6 *)
+			    &peer->sa_local)->sin6_addr,
+			    sizeof(struct in6_addr)) == -1) {
+				log_warnx("mrt_dump_bgp_msg: buf_add error");
+				buf_free(buf);
+				return (-1);
+			}
 		break;
 	}
 
@@ -181,8 +206,8 @@ mrt_dump_state(struct mrt *mrt, u_int16_t old_state, u_int16_t new_state,
 		return (-1);
 	}
 
-	DUMP_SHORT(buf, bgp->as);
-	DUMP_SHORT(buf, peer->conf.remote_as);
+	DUMP_SHORT(buf, bgp->short_as);
+	DUMP_SHORT(buf, peer->short_as);
 	DUMP_SHORT(buf, /* ifindex */ 0);
 	switch (peer->sa_local.ss_family) {
 	case AF_INET:
@@ -351,7 +376,7 @@ mrt_dump_entry_mp(struct mrt *mrt, struct prefix *p, u_int16_t snum,
 	}
 
 	DUMP_SHORT(buf, rde_local_as());
-	DUMP_SHORT(buf, peer->conf.remote_as);
+	DUMP_SHORT(buf, peer->short_as);
 	DUMP_SHORT(buf, /* ifindex */ 0);
 
 	switch (af) {
@@ -404,25 +429,25 @@ mrt_dump_entry_mp(struct mrt *mrt, struct prefix *p, u_int16_t snum,
 	}
 
 	if ((bptr = buf_reserve(buf, p_len)) == NULL) {
-		log_warnx("mrt_dump_entry_mpbuf_reserve error");
+		log_warnx("mrt_dump_entry_mp: buf_reserve error");
 		buf_free(buf);
 		return (-1);
 	}
 	if (prefix_write(bptr, p_len, &addr, p->prefix->prefixlen) == -1) {
-		log_warnx("mrt_dump_entry_mpprefix_write error");
+		log_warnx("mrt_dump_entry_mp: prefix_write error");
 		buf_free(buf);
 		return (-1);
 	}
 
 	DUMP_SHORT(buf, attr_len);
 	if ((bptr = buf_reserve(buf, attr_len)) == NULL) {
-		log_warnx("mrt_dump_entry_mpbuf_reserve error");
+		log_warnx("mrt_dump_entry_mp: buf_reserve error");
 		buf_free(buf);
 		return (-1);
 	}
 
 	if (mrt_attr_dump(bptr, attr_len, p->aspath, NULL) == -1) {
-		log_warnx("mrt_dump_entry_mpmrt_attr_dump error");
+		log_warnx("mrt_dump_entry_mp: mrt_attr_dump error");
 		buf_free(buf);
 		return (-1);
 	}
@@ -467,7 +492,7 @@ mrt_dump_entry(struct mrt *mrt, struct prefix *p, u_int16_t snum,
 	DUMP_BYTE(buf, 1);		/* state */
 	DUMP_LONG(buf, p->lastchange);	/* originated */
 	DUMP_NLONG(buf, peer->remote_addr.v4.s_addr);
-	DUMP_SHORT(buf, peer->conf.remote_as);
+	DUMP_SHORT(buf, peer->short_as);
 
 	DUMP_SHORT(buf, attr_len);
 	if ((bptr = buf_reserve(buf, attr_len)) == NULL) {
