@@ -1,4 +1,4 @@
-/*	$OpenBSD: rde.c,v 1.230 2008/02/26 19:58:51 claudio Exp $ */
+/*	$OpenBSD: rde.c,v 1.232 2008/06/15 10:03:46 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -748,6 +748,11 @@ rde_update_dispatch(struct imsg *imsg)
 
 	p = imsg->data;
 
+	if (imsg->hdr.len < IMSG_HEADER_SIZE + 2) {
+		rde_update_err(peer, ERR_UPDATE, ERR_UPD_ATTRLIST, NULL, 0);
+		return (-1);
+	}
+
 	memcpy(&len, p, 2);
 	withdrawn_len = ntohs(len);
 	p += 2;
@@ -1717,7 +1722,7 @@ rde_dump_rib_as(struct prefix *p, struct rde_aspath *asp, pid_t pid, int flags)
 		rib.flags |= F_RIB_INTERNAL;
 	if (asp->flags & F_PREFIX_ANNOUNCED)
 		rib.flags |= F_RIB_ANNOUNCE;
-	if (asp->nexthop != NULL && asp->nexthop->state == NEXTHOP_REACH)
+	if (asp->nexthop == NULL || asp->nexthop->state == NEXTHOP_REACH)
 		rib.flags |= F_RIB_ELIGIBLE;
 	if (asp->flags & F_ATTR_LOOP)
 		rib.flags &= ~F_RIB_ELIGIBLE;
@@ -2846,4 +2851,3 @@ sa_cmp(struct bgpd_addr *a, struct sockaddr *b)
 
 	return (0);
 }
-
