@@ -1,4 +1,4 @@
-/*	$OpenBSD: rde_rib.c,v 1.83 2006/01/24 13:34:33 claudio Exp $ */
+/*	$OpenBSD: rde_rib.c,v 1.96 2007/06/01 04:17:30 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Claudio Jeker <claudio@openbsd.org>
@@ -30,7 +30,7 @@
  * BGP RIB -- Routing Information Base
  *
  * The RIB is build with one aspect in mind. Speed -- actually update speed.
- * Therefor one thing needs to be absolutely avoided, long table walks.
+ * Therefore one thing needs to be absolutely avoided, long table walks.
  * This is achieved by heavily linking the different parts together.
  */
 
@@ -90,7 +90,6 @@ path_update(struct rde_peer *peer, struct rde_aspath *nasp,
 	struct rde_aspath	*asp;
 	struct prefix		*p, *oldp = NULL;
 
-	
 	if (flags & F_LOCAL) {
 		rde_send_pftable(nasp->pftableid, prefix, prefixlen, 0);
 		rde_send_pftable_commit();
@@ -137,7 +136,7 @@ path_update(struct rde_peer *peer, struct rde_aspath *nasp,
 	}
 
 	/* Do not try to move a prefix that is in the wrong RIB. */
-	if (p != NULL && (p->flags & flags) == 0)
+	if (p == NULL || (p->flags & flags) == 0)
 		p = oldp;
 
 	/*
@@ -227,24 +226,15 @@ path_remove(struct rde_aspath *asp)
 	while ((p = LIST_FIRST(&asp->prefix_h)) != NULL) {
 		/* Commit is done in peer_down() */
 		pt_getaddr(p->prefix, &addr);
-		rde_send_pftable(p->aspath->pftableid,
-		    &addr, p->prefix->prefixlen, 1);
+		if (p->flags & F_LOCAL)
+			rde_send_pftable(p->aspath->pftableid, &addr,
+			    p->prefix->prefixlen, 1);
 
 		prefix_destroy(p);
 	}
 	path_destroy(asp);
 }
 
-void
-path_updateall(struct rde_aspath *asp, enum nexthop_state state)
-{
-	if (rde_noevaluate())
-		/* if the decision process is turned off this is a no-op */
-		return;
-
-	prefix_updateall(asp, state);
-}
-
 /* this function is only called by prefix_remove and path_remove */
 void
 path_destroy(struct rde_aspath *asp)
@@ -292,7 +282,7 @@ path_link(struct rde_aspath *asp, struct rde_peer *peer)
 }
 
 /*
- * copy asp to a new UNLINKED one manly for filtering
+ * copy asp to a new UNLINKED one mainly for filtering
  */
 struct rde_aspath *
 path_copy(struct rde_aspath *asp)
@@ -312,6 +302,8 @@ path_copy(struct rde_aspath *asp)
 	nasp->origin = asp->origin;
 	nasp->rtlabelid = asp->rtlabelid;
 	rtlabel_ref(nasp->rtlabelid);
+	nasp->pftableid = asp->pftableid;
+	pftable_ref(nasp->pftableid);
 
 	nasp->flags = asp->flags & ~F_ATTR_LINKED;
 	attr_copy(nasp, asp);
@@ -340,7 +332,7 @@ path_get(void)
 	return (asp);
 }
 
-/* free a unlinked element */
+/* free an unlinked element */
 void
 path_put(struct rde_aspath *asp)
 {
@@ -351,6 +343,7 @@ path_put(struct rde_aspath *asp)
 		fatalx("path_put: linked object");
 
 	rtlabel_unref(asp->rtlabelid);
+	pftable_unref(asp->pftableid);
 	aspath_put(asp->aspath);
 	attr_freeall(asp);
 	rdemem.path_cnt--;
@@ -387,6 +380,8 @@ prefix_compare(const struct bgpd_addr *a, const struct bgpd_addr *b,
 			return (aa - ba);
 		return (0);
 	case AF_INET6:
+		if (prefixlen > 128)
+			fatalx("prefix_cmp: bad IPv6 prefixlen");
 		for (i = 0; i < prefixlen / 8; i++)
 			if (a->v6.s6_addr[i] != b->v6.s6_addr[i])
 				return (a->v6.s6_addr[i] - b->v6.s6_addr[i]);
@@ -642,7 +637,7 @@ prefix_updateall(struct rde_aspath *asp, enum nexthop_state state)
 		/*
 		 * If the prefix is the active one remove it first,
 		 * this has to be done because we can not detect when
-		 * the active prefix changes it's state. In this case
+		 * the active prefix changes its state. In this case
 		 * we know that this is a withdrawl and so the second
 		 * prefix_evaluate() will generate no update because
 		 * the nexthop is unreachable or ineligible.
@@ -771,7 +766,7 @@ struct nexthop		*nexthop_lookup(struct bgpd_addr *);
  * BGP and the true nexthop which is used in the FIB -- forward information
  * base a.k.a kernel routing table. When sending updates it is even more
  * confusing. In IBGP we pass the unmodified exit nexthop to the neighbors
- * while in EBGP normaly the address of the router is sent. The exit nexthop
+ * while in EBGP normally the address of the router is sent. The exit nexthop
  * may be passed to the external neighbor if the neighbor and the exit nexthop
  * reside in the same subnet -- directly connected.
  */
@@ -801,10 +796,18 @@ void
 nexthop_shutdown(void)
 {
 	u_int32_t		 i;
+	struct nexthop		*nh, *nnh;
 
-	for (i = 0; i <= nexthoptable.nexthop_hashmask; i++)
+	for (i = 0; i <= nexthoptable.nexthop_hashmask; i++) {
+		for (nh = LIST_FIRST(&nexthoptable.nexthop_hashtbl[i]);
+		    nh != NULL; nh = nnh) {
+			nnh = LIST_NEXT(nh, nexthop_l);
+			nh->state = NEXTHOP_UNREACH;
+			(void)nexthop_delete(nh);
+		}
 		if (!LIST_EMPTY(&nexthoptable.nexthop_hashtbl[i]))
 			log_warnx("nexthop_shutdown: non-free table");
+	}
 
 	free(nexthoptable.nexthop_hashtbl);
 }
@@ -817,7 +820,8 @@ nexthop_update(struct kroute_nexthop *msg)
 
 	nh = nexthop_lookup(&msg->nexthop);
 	if (nh == NULL) {
-		log_warnx("nexthop_update: non-existent nexthop");
+		log_warnx("nexthop_update: non-existent nexthop %s",
+		    log_addr(&msg->nexthop));
 		return;
 	}
 
@@ -826,6 +830,10 @@ nexthop_update(struct kroute_nexthop *msg)
 	else
 		nh->state = NEXTHOP_UNREACH;
 
+	if (nexthop_delete(nh))
+		/* nexthop no longer used */
+		return;
+
 	if (msg->connected) {
 		nh->flags |= NEXTHOP_CONNECTED;
 		memcpy(&nh->true_nexthop, &nh->exit_nexthop,
@@ -858,7 +866,7 @@ nexthop_update(struct kroute_nexthop *msg)
 		return;
 
 	LIST_FOREACH(asp, &nh->path_h, nexthop_l) {
-		path_updateall(asp, nh->state);
+		prefix_updateall(asp, nh->state);
 	}
 }
 
@@ -880,6 +888,10 @@ nexthop_modify(struct rde_aspath *asp, struct bgpd_addr *nexthop,
 		asp->flags |= F_NEXTHOP_NOMODIFY;
 		return;
 	}
+	if (type == ACTION_SET_NEXTHOP_SELF) {
+		asp->flags |= F_NEXTHOP_SELF;
+		return;
+	}
 	if (af != nexthop->af)
 		return;
 
@@ -914,13 +926,26 @@ nexthop_unlink(struct rde_aspath *asp)
 	nh = asp->nexthop;
 	asp->nexthop = NULL;
 
-	if (LIST_EMPTY(&nh->path_h)) {
-		LIST_REMOVE(nh, nexthop_l);
-		rde_send_nexthop(&nh->exit_nexthop, 0);
+	(void)nexthop_delete(nh);
+}
 
-		rdemem.nexthop_cnt--;
-		free(nh);
-	}
+int
+nexthop_delete(struct nexthop *nh)
+{
+	/* nexthop still used by some other aspath */
+	if (!LIST_EMPTY(&nh->path_h))
+		return (0);
+
+	/* either pinned or in a state where it may not be deleted */
+	if (nh->refcnt > 0 || nh->state == NEXTHOP_LOOKUP)
+		return (0);
+
+	LIST_REMOVE(nh, nexthop_l);
+	rde_send_nexthop(&nh->exit_nexthop, 0);
+
+	rdemem.nexthop_cnt--;
+	free(nh);
+	return (1);
 }
 
 struct nexthop *
@@ -952,7 +977,7 @@ nexthop_compare(struct nexthop *na, struct nexthop *nb)
 {
 	struct bgpd_addr	*a, *b;
 
-	if (na == NULL && nb == NULL)
+	if (na == nb)
 		return (0);
 	if (na == NULL)
 		return (-1);
